package com.tzld.longarticle.recommend.server.service.recommend;

import com.alibaba.fastjson.JSONArray;
import com.tzld.longarticle.recommend.server.common.enums.aigc.PublishContentStatusEnum;
import com.tzld.longarticle.recommend.server.common.enums.longArticle.ArticleVideoAuditStatusEnum;
import com.tzld.longarticle.recommend.server.mapper.aigc.PublishContentMapper;
import com.tzld.longarticle.recommend.server.mapper.longArticle.ArticleAuditMapper;
import com.tzld.longarticle.recommend.server.model.dto.PublishContentMiniprogramDTO;
import com.tzld.longarticle.recommend.server.model.entity.longArticle.ArticleReMatchRecord;
import com.tzld.longarticle.recommend.server.model.entity.longArticle.LongArticleCrawlerVideo;
import com.tzld.longarticle.recommend.server.model.entity.longArticle.LongArticleTitleAudit;
import com.tzld.longarticle.recommend.server.model.entity.longArticle.LongArticlesMatchVideo;
import com.tzld.longarticle.recommend.server.model.param.videoAudit.ArticleAuditParam;
import com.tzld.longarticle.recommend.server.model.param.videoAudit.ArticleVideoAuditListParam;
import com.tzld.longarticle.recommend.server.model.param.videoAudit.AuditFilterParam;
import com.tzld.longarticle.recommend.server.model.param.videoAudit.VideoAuditParam;
import com.tzld.longarticle.recommend.server.model.vo.ArticleVideoAuditListVO;
import com.tzld.longarticle.recommend.server.repository.longArticle.*;
import com.tzld.longarticle.recommend.server.util.page.Page;
import com.xxl.job.core.biz.model.ReturnT;
import com.xxl.job.core.handler.annotation.XxlJob;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections4.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import java.util.*;
import java.util.stream.Collectors;

@Service
@Slf4j
public class ArticleVideoAuditService {

    @Autowired
    private ArticleAuditMapper articleAuditMapper;
    @Autowired
    private PublishContentMapper publishContentMapper;
    @Autowired
    private ArticleReMatchRecordRepository articleReMatchRecordRepository;
    @Autowired
    private LongArticleCrawlerVideoRepository crawlerVideoRepository;
    @Autowired
    private LongArticleTitleAuditRepository titleAuditRepository;
    @Autowired
    private LongArticlesMatchVideoRepository articlesMatchVideoRepository;
    @Autowired
    private ArticleAuditService articleAuditService;

    @Value("${cdnUrl:https://rescdn.piaoquantv.com/}")
    private String cdnUrl;

    public Page<ArticleVideoAuditListVO> list(ArticleVideoAuditListParam param) {
        int offset = (param.getPageNum() - 1) * param.getPageSize();
        if (CollectionUtils.isEmpty(param.getStatus())) {
            param.setStatus(Collections.singletonList(ArticleVideoAuditStatusEnum.WAITING.getCode()));
        }
        int count = articleAuditMapper.articleVideoAuditListCount(param.getContentId(), param.getStatus(),
                param.getTitle(), param.getAuditAccount());
        List<ArticleVideoAuditListVO> list = articleAuditMapper.articleVideoAuditList(param.getContentId(),
                param.getStatus(), param.getTitle(), param.getAuditAccount(), offset, param.getPageSize());
        buildArticleVideoAuditListVO(list);
        Page<ArticleVideoAuditListVO> page = new Page<>(param.getPageNum(), param.getPageSize());
        page.setTotalSize(count);
        page.setObjs(list);
        return page;
    }

    private void buildArticleVideoAuditListVO(List<ArticleVideoAuditListVO> list) {
        if (CollectionUtils.isEmpty(list)) {
            return;
        }
        List<String> contentIds = list.stream().map(ArticleVideoAuditListVO::getContentId).collect(Collectors.toList());
        List<LongArticleCrawlerVideo> videoList = crawlerVideoRepository.getByContentIdInAndIsIllegal(contentIds, 0);
        Map<String, List<LongArticleCrawlerVideo>> videoMap = videoList.stream()
                .collect(Collectors.groupingBy(LongArticleCrawlerVideo::getContentId));
        for (ArticleVideoAuditListVO item : list) {
            List<LongArticleCrawlerVideo> videos = videoMap.get(item.getContentId());
            if (CollectionUtils.isEmpty(videos)) {
                continue;
            }
            List<ArticleVideoAuditListVO.VideoItem> videoItems = videos.stream().map(video -> {
                ArticleVideoAuditListVO.VideoItem videoItem = new ArticleVideoAuditListVO.VideoItem();
                videoItem.setVideoId(video.getId());
                videoItem.setTitle(video.getVideoTitle());
                videoItem.setVideoUrl(cdnUrl + video.getVideoOssPath());
                if (StringUtils.hasText(video.getCoverOssPath())) {
                    videoItem.setCoverUrl(cdnUrl + video.getCoverOssPath());
                } else {
                    videoItem.setCoverUrl(video.getCoverUrl());
                }
                videoItem.setStatus(video.getStatus());
                videoItem.setAuditAccount(video.getAuditAccount());
                videoItem.setAuditTimestamp(video.getAuditTimestamp());
                return videoItem;
            }).collect(Collectors.toList());
            item.setVideoList(videoItems);
            item.setVideoCount(videoItems.size());
            item.setWaitingAuditVideoCount(videoItems.size());
        }
    }

    public void auditArticle(ArticleAuditParam param) {
        LongArticleTitleAudit titleAudit = titleAuditRepository.getByContentId(param.getContentId());
        Long now = System.currentTimeMillis();
        titleAudit.setStatus(param.getStatus());
        titleAudit.setAuditAccount(param.getAuditAccount());
        titleAudit.setAuditTimestamp(now);
        titleAuditRepository.save(titleAudit);
        if (param.getStatus() == 2) {
            // 审核不通过
            auditArticleReject(titleAudit);
        } else {
            // 审核通过
            auditArticlePass(titleAudit);
        }

    }

    private void auditArticlePass(LongArticleTitleAudit titleAudit) {
        // publish_content
        publishContentMapper.updatePublishContentStatus(titleAudit.getContentId(),
                PublishContentStatusEnum.waiting_publish.getVal(),
                PublishContentStatusEnum.gzh_article_deprecated.getVal());
    }

    private void auditArticleReject(LongArticleTitleAudit titleAudit) {
        // reMatch 暂时取消
//        List<LongArticleCrawlerVideo> videoList = crawlerVideoRepository.getByContentIdAndIsIllegal(titleAudit.getContentId(), 1);
//        if (CollectionUtils.isNotEmpty(videoList)) {
//            for (LongArticleCrawlerVideo video : videoList) {
//                ArticleReMatchRecord reMatchRecord = new ArticleReMatchRecord();
//                reMatchRecord.setContentId(titleAudit.getContentId());
//                reMatchRecord.setOssPath(video.getVideoOssPath());
//                reMatchRecord.setStatus(0);
//                reMatchRecord.setCreateTimestamp(System.currentTimeMillis() / 1000);
//                articleReMatchRecordRepository.save(reMatchRecord);
//            }
//        }
        // publish_content
        publishContentMapper.updatePublishContentStatus(titleAudit.getContentId(),
                PublishContentStatusEnum.gzh_article_deprecated.getVal(),
                PublishContentStatusEnum.waiting_publish.getVal());
    }

    public void auditVideo(VideoAuditParam param) {
        LongArticleCrawlerVideo video = crawlerVideoRepository.getById(param.getVideoId());
        Long now = System.currentTimeMillis();
        video.setStatus(param.getStatus());
        video.setAuditAccount(param.getAuditAccount());
        video.setAuditTimestamp(now);
        if (param.getStatus() == 2) {
            video.setIsIllegal(1);
            // 审核不通过
            auditVideoReject(video);
        } else {
            video.setIsIllegal(0);
        }
        crawlerVideoRepository.save(video);
    }

    private void auditVideoReject(LongArticleCrawlerVideo video) {
        // 已发布文章删除
        List<LongArticlesMatchVideo> matchVideos = articlesMatchVideoRepository.getByContentId(video.getContentId());
        List<String> traceIds = new ArrayList<>();
        if (CollectionUtils.isNotEmpty(matchVideos)) {
            for (LongArticlesMatchVideo matchVideo : matchVideos) {
                // 发布位置为头条
                List<String> results = JSONArray.parseArray(matchVideo.getResponse(), String.class);
                if (CollectionUtils.isNotEmpty(results) && results.get(0).contains(video.getVideoOssPath())) {
                    traceIds.add(matchVideo.getTraceId());
                }
            }
        }
        if (CollectionUtils.isEmpty(traceIds)) {
            return;
        }
        List<PublishContentMiniprogramDTO> publishContents = publishContentMapper.getPublishContentByTraceIdIn(traceIds);
        List<String> publishContentIds = publishContents.stream().map(PublishContentMiniprogramDTO::getPublishContentId)
                .collect(Collectors.toList());
        articleAuditService.buildArticleAuditDelete(publishContentIds);
    }

    public List<String> getFilterValue(AuditFilterParam param) {
        return articleAuditMapper.searchFilterValueByItemName(param.getItemName(), param.getSearchKeyword());
    }

    @XxlJob("rematchVideoAudit")
    public ReturnT<String> rematchVideoAudit(String param) {
        List<LongArticleTitleAudit> rejectList = titleAuditRepository.getByStatus(ArticleVideoAuditStatusEnum.REJECT.getCode());
        if (CollectionUtils.isEmpty(rejectList)) {
            return ReturnT.SUCCESS;
        }
        List<String> contentIds = rejectList.stream().map(LongArticleTitleAudit::getContentId).collect(Collectors.toList());
        List<ArticleReMatchRecord> reMatchRecordList = articleReMatchRecordRepository.getByContentIdIn(contentIds);
        if (CollectionUtils.isEmpty(reMatchRecordList)) {
            return ReturnT.SUCCESS;
        }
        Map<String, List<ArticleReMatchRecord>> reMatchRecordMap = reMatchRecordList.stream()
               .collect(Collectors.groupingBy(ArticleReMatchRecord::getContentId));
        for (LongArticleTitleAudit audit : rejectList) {
            List<ArticleReMatchRecord> records = reMatchRecordMap.get(audit.getContentId());
            if (CollectionUtils.isEmpty(records)) {
                continue;
            }
            records = records.stream().filter(o -> Objects.isNull(o.getTraceId())).collect(Collectors.toList());
            if (CollectionUtils.isEmpty(records)) {
                continue;
            }
            List<ArticleReMatchRecord> failList = records.stream()
                    .filter(o -> o.getStatus().equals(2) || o.getStatus().equals(0)).collect(Collectors.toList());
            if (CollectionUtils.isNotEmpty(failList)) {
                continue;
            }
            audit.setStatus(ArticleVideoAuditStatusEnum.WAITING.getCode());
            titleAuditRepository.save(audit);
        }
        return ReturnT.SUCCESS;
    }
}
